
#include "proto.h"

extern int iMaster;

i2c_status_t i2c_slave_send(i2c_dev_t *dev)
{
	int status, timeout;
	if ( uiVerbosity >= LOGGING_DEBUG_FINE )
		slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_DEBUG2, "%s:i2c_slave_send: entry", pszProgram);
	char *buf = dev->slave.data;
	int len = dev->slave.len;
	/* At this point we have been addressed as a slave
	 * and we replied with an ack
	 * now we start to send data
	 */

//	out16(dev->regbase + I2C_CTRREG_OFF, AM335x_I2C_EN | AM335x_TRX );  // Mark as Slave Transmitter
	printf( "Set Control register 0x%x\n", in16(dev->regbase + I2C_CTRREG_OFF) );
	/* Send the rest of the data */
//	out16(dev->regbase + AM335x_I2C_SBLOCK, 0 );					// Tell it to only use OA0
	int iCount = 1;
	while (len > 0)
	{
		timeout = 2000;
		while ( --timeout && ( ( status = in16(dev->regbase + I2C_STSREG_OFF) ) & AM335x_I2C_SLV_XMT ) != AM335x_I2C_SLV_XMT && !( status & ( AM335x_I2C_ERROR ) ) ); //| AM335x_I2C_ARDY	) ) );
		if ( !timeout )
		{
			slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_ERROR, "%s:i2c_slave_send: timeout XMT status 0x%X, so reset", pszProgram, status );
			i2c_reset(dev);
			return -1;
		}
		if ( iMaster )
			return -1;
		if (status & AM335x_I2C_ERROR)
		{
			if (uiVerbosity >= LOGGING_ERRORS)
				slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_WARNING, "%s:i2c_slave_send: error return 0x%X", pszProgram, status);
			i2c_reset(dev);
			return status;
		}
		if ((status & ( AM335x_I2C_SLV_XMT ) ) == AM335x_I2C_SLV_XMT)
		{
			out16(dev->regbase + I2C_DATREG_OFF, (*(char *)buf) & 0x0FF);
			out16(dev->regbase + AM335x_I2C_STAT, AM335x_I2C_XRDY );
			if ( uiVerbosity >= LOGGING_PROGRESS )
				slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_INFO, "%s:i2c_slave_send: Byte %d is 0x%02X '%s', status 0x%X",
						pszProgram, iCount, *((unsigned char *)buf), fnlibCharToString( *((char *)buf) ), status);
//		    printf( "Slave write Byte %d is 0x%02X, status 0x%X\n", iCount, *((unsigned char *)buf), status );
			++buf; --len, iCount++;
		}
/*
		if ( status & AM335x_I2C_ARDY )
		{
			if (uiVerbosity >= LOGGING_ERRORS)
				slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_WARNING, "%s:i2c_slave_send: ARDY found before complete return 0x%X", pszProgram, status);
			out16(dev->regbase + AM335x_I2C_STAT, AM335x_I2C_ARDY );
			return status;
		}
*/
	}
	timeout = 2000;
	while ( !( in16(dev->regbase + I2C_STSREG_OFF) & AM335x_I2C_ARDY ) && --timeout );
	if ( timeout )
	{
//		printf( "Slave write wrote %d bytes, status 0x%X\n", dev->slave.len, in16(dev->regbase + I2C_STSREG_OFF) );
		return 0;
	}

	while( !( ( status = in16(dev->regbase + I2C_STSREG_OFF) ) & ( AM335x_I2C_ARDY ) ) && !iMaster && ( status & AM335x_I2C_BB ) ) // | AM335x_I2C_BF ) ) )
	{
		if (status & AM335x_I2C_ERROR)
		{
			if (uiVerbosity >= LOGGING_ERRORS)
				slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_WARNING, "%s:i2c_slave_send: error return 0x%X", pszProgram, status);
			i2c_reset(dev);
			return status;
		}
		if ((status & ( AM335x_I2C_SLV_XMT ) ) == AM335x_I2C_SLV_XMT)
		{
			len++;
			out16(dev->regbase + I2C_DATREG_OFF, 0x00 );
			out16(dev->regbase + AM335x_I2C_STAT, AM335x_I2C_XRDY ); // | AM335x_I2C_XUDF );
			if ( uiVerbosity >= LOGGING_DEBUG_FINE )
				slogf(_SLOGC_SNELL_IQMIT_I2C_DRIVER, _SLOG_DEBUG2,
						"%s:i2c_slave_send: pad length %d, status 0x%X", pszProgram, len, status);
//			printf( "Pad %d, count %d, status 0x%X\n", len, iCount, status );
			iCount++;
		}
	}
	return status;
}
